@@ -14,7 +14,8 @@ from Accuinsight.modeler.core.LcConst.LcConst import RUN_NAME_KERAS
 from Accuinsight.modeler.core.get_for_visual import roc_pr_curve, get_visual_info_regressor
 from Accuinsight.modeler.core.Run.RunInfo.RunInfo import set_current_runs, clear_runs, set_model_json_path, \
     set_visual_csv_path, set_visual_json_path, set_best_model_h5_path, set_best_model_json_path, \
-    set_python_dependencies, set_run_name, set_model_file_path, set_prefix_path
+    set_python_dependencies, set_run_name, set_model_file_path, set_prefix_path, set_shap_json_path
+from Accuinsight.modeler.core.feature_contribution import shap_value
 from Accuinsight.modeler.utils.dl_utils import delete_files_except_best, get_best_model_path
 from Accuinsight.modeler.utils.dependency.dependencies import gather_sources_and_dependencies
 from Accuinsight.modeler.utils.os_getenv import is_in_ipython, get_current_notebook, get_os_env
@@ -31,8 +32,11 @@ class accuinsight(Common):
     def __init__(self):
         super().__init__()
 
-    def autolog(self, tag=None, best_weights=False):
-        global description, endpoint, var_model_file_path, message, thresholds, token, channel_id
+    def get_file(self, storage_json_file_name=None):
+        super().get_file(storage_json_file_name)
+
+    def autolog(self, tag=None, best_weights=False, model_monitor=False):
+        global description, endpoint, var_model_file_path, message, thresholds, token, channel_id, run_id, shap_on
         description = tag
         endpoint = self.endpoint
         message = self.message
@@ -44,6 +48,20 @@ class accuinsight(Common):
         else:
             best_weights_on = False
 
+        if model_monitor:
+            shap_on = True
+            if self.feature_name is None:
+                try:
+                    feature_name = get.feature_name(self.save_path, self.StorageInfo, self.target_name)
+
+                except:
+                    pass      # when user did not use <get_file> or <set_feature> function
+            else:
+                feature_name = self.feature_name
+        else:
+            shap_on = False
+        run_id = None
+
         # if in notebook
         if is_in_ipython():
             var_model_file_path = get_current_notebook()
@@ -67,7 +85,7 @@ class accuinsight(Common):
                 save_git_info=True
             )
             var_model_file_path = mainfile['filename']
-    
+
         class TrainHistoryCallbacks(keras.callbacks.Callback):
             def __init__(self, verbose=1, mode='auto', period=1):
                 super(TrainHistoryCallbacks, self).__init__()
@@ -79,13 +97,11 @@ class accuinsight(Common):
                 self.model_summary = OrderedDict()
                 self.run_id = None
                 self.model_type = None
-        
+
             def on_train_begin(self, logs={}):
-                if best_weights_on:
-                    logging.info('Using autolog(best_weights=True)')
-                else:
-                    logging.info('Using autolog(best_weights=False)')
-            
+                logging.info('Using autolog(best_weights={}, model_monitor={}'
+                             .format(str(best_weights_on), str(shap_on)))
+
                 global start
                 start = get_time.now()
                 self.run_id = func.get_run_id()
@@ -96,28 +112,28 @@ class accuinsight(Common):
                 self.model_summary['logging_time'] = get_time.logging_time()
                 self.model_summary['run_id'] = func.get_run_id()
                 self.model_summary['model_type'] = get.model_type(self.model)
-                
+
                 if hasattr(self.model.loss, 'get_config'):
                     self.model_summary['loss_function'] = self.model.loss.get_config()['name']
                 else:
                     self.model_summary['loss_function'] = self.model.loss
-                
+
                 self.model_summary['optimizer_info'] = {type(self.model.optimizer).__name__: self.model.optimizer.get_config()}
 
                 '''[get best model] on_train_begin '''
                 self.best_weights = self.model.get_weights()
-              
+
                 self.dict_path = path.get_file_path(self.model, usedFramework='keras')
-            
+
                 set_prefix_path(self.dict_path[LcConst.RUN_PREFIX_PATH])
-            
+
                 set_run_name(self.model_type, self.run_id)
                 set_python_dependencies(py_depenpency=dependencies)
-            
+
             '''[get best model] on_epoch_end '''
             def on_epoch_end(self, epoch, logs=None):
                 logs = logs or {}
-                
+
                 if epoch == 0:
                     if len(self.model.metrics_names) == 1 and 'loss' in self.model.metrics_names:
                         self.monitor = 'val_loss'
@@ -154,55 +170,38 @@ class accuinsight(Common):
                         current = logs.get(self.monitor)
                         if current is None:
                             warnings.warn('Can pick best model only with %s available, '
-                                      'skipping.' % (self.monitor), RuntimeWarning)
+                                          'skipping.' % (self.monitor), RuntimeWarning)
                         else:
                             if self.monitor_op(current, self.best):
-#                                 if self.verbose > 0:
-#                                     print('\n\nEpoch %05d: %s improved from %0.5f to %0.5f,'
-#                                           ' storing weights.\n'
-#                                           % (epoch + 1, self.monitor, self.best, current))
                                 self.best = current
                                 self.best_epochs = epoch + 1
                                 self.best_weights = self.model.get_weights()
                             else:
                                 pass
-#                                 if self.verbose > 0:
-#                                     print('\n\nEpoch %05d: %s did not improve\n' % (epoch + 1, self.monitor))
 
                     self.current_value = current
-                    
-                    # model save path
-                    common_path = self.dict_path['save_model_path'] + self.model_summary['model_type'] + '-' + \
-                    self.model_summary['run_id'] + '-epoch-' + str(epoch + 1).zfill(5) + '-' + self.monitor + '-' + str(current).zfill(5)
-                    save_model_path = common_path + '.json'
-                    save_weights_path = common_path + '.h5'
 
-                    # model to JSON
-                    model_json = self.model.to_json()
-                    with open(save_model_path, "w") as json_file:
-                        json_file.write(model_json)
-                    # weights to H5
-                    self.model.save_weights(save_weights_path)
-    
                 # Not using best_weights
                 else:
-                    self.last_epochs = epoch + 1
                     self.last_epoch_metric = logs.get(self.monitor)
-                    
-                    # model save path
-                    common_path = self.dict_path['save_model_path'] + self.model_summary['model_type'] + '-' + \
-                    self.model_summary['run_id'] + '-epoch-' + str(epoch + 1).zfill(5) + '-' + self.monitor + '-' + str(self.last_epoch_metric).zfill(5)
-                    save_model_path = common_path + '.json'
-                    save_weights_path = common_path + '.h5'
+                    self.best_epochs = epoch + 1
+                    current = logs.get(self.monitor)
+                    self.current_value = current
 
-                    # model to JSON
-                    model_json = self.model.to_json()
-                    with open(save_model_path, "w") as json_file:
-                        json_file.write(model_json)
-                    # weights to H5
-                    self.model.save_weights(save_weights_path)
+                # model save path
+                run_id = self.model_summary['model_type'] + '-' + self.model_summary['run_id']
+                common_path = self.dict_path['save_model_path'] + run_id + '-epoch-' + str(epoch + 1).zfill(5) \
+                              + '-' + self.monitor + '-' + str(current).zfill(5)
+                save_model_path = common_path + '.json'
+                save_weights_path = common_path + '.h5'
+
+                # model to JSON
+                model_json = self.model.to_json()
+                with open(save_model_path, "w") as json_file:
+                    json_file.write(model_json)
                     
-                    self.current_value = self.last_epoch_metric
+                # weights to H5
+                self.model.save_weights(save_weights_path)
             
             def on_train_end(self, logs={}):
                 '''[get best model] on_train_end '''
@@ -244,22 +243,22 @@ class accuinsight(Common):
                     delete_files_except_best(run_id=self.model_summary['run_id'], epochs=str(self.last_epochs),
                     path=self.dict_path)
 
-                path_for_setting_model_json = self.dict_path['save_model_dir'] + get_best_model_path(run_id = self.run_id, path = self.dict_path)['json']
+                path_for_setting_model_json = self.dict_path['save_model_dir'] + get_best_model_path(run_id=self.model_summary['run_id'], path=self.dict_path)['json']
                 set_best_model_json_path(path_for_setting_model_json)
-                path_for_setting_model_h5 = self.dict_path['save_model_dir'] + get_best_model_path(run_id = self.run_id, path = self.dict_path)['h5']
+                path_for_setting_model_h5 = self.dict_path['save_model_dir'] + get_best_model_path(run_id=self.model_summary['run_id'], path=self.dict_path)['h5']
                 set_best_model_h5_path(path_for_setting_model_h5)
 
                 start_ts = int(start.timestamp())
                 end_ts = int(end.timestamp())
                 delta_ts = end_ts - start_ts
-            
+
                 clear_runs(start_ts, end_ts, delta_ts)
                 accuinsight._send_message(metric=self.monitor,
                                           current_value=self.current_value,
-                                          message=self.message,
-                                          thresholds=self.thresholds,
-                                          token=self.token,
-                                          channel_id=self.channel_id)
+                                          message=message,
+                                          thresholds=thresholds,
+                                          token=token,
+                                          channel_id=channel_id)
                 env_value = get_os_env('ENV')
                 modeler_rest = LifecycleRestApi(env_value[LcConst.BACK_END_API_URL],
                                                 env_value[LcConst.BACK_END_API_PORT],
@@ -296,13 +295,37 @@ class accuinsight(Common):
 
                     with open(visual_json_full_path, 'w', encoding='utf-8') as save_file:
                         json.dump(visual_regression_json, save_file, indent="\t")
-            
-        def run_and_log_function(self, original, args, kwargs, unlogged_params):
+
+        class shapCallbacks(keras.callbacks.Callback):
+            def __init__(self, trainX, feature_name, run_id, trigger=shap_on):
+                super(shapCallbacks, self).__init__()
+                self.trainX = trainX
+                self.trigger = trigger
+                self.run_id = run_id
+                self.feature_name_in_shap = feature_name
+
+            def on_train_end(self, logs={}):
+                if self.trigger:
+                    self.shap_value = shap_value(self.model, self.trainX, self.feature_name_in_shap)
+
+                    self.dict_path = path.get_file_path(self.model, usedFramework='keras')
+
+                    # path for shap.json
+                    shap_json_full_path = self.dict_path['shap_json_full']
+                    set_shap_json_path(self.dict_path['shap_json'])
+
+                    with open(shap_json_full_path, 'w', encoding='utf-8') as save_file:
+                        json.dump(self.shap_value, save_file, indent='\t')
+
+                else:
+                    pass
+
+        def run_and_log_function(self, original, x, y, kwargs, unlogged_params):
             dict_path = path.get_file_path(self, usedFramework='keras')
-        
+
             path_for_setting_visual_csv = dict_path['visual_csv']
             visual_csv_full_path = dict_path['visual_csv_full']
-        
+
             # set current run
             set_current_runs(RUN_NAME_KERAS)
             set_model_file_path(var_model_file_path)
@@ -310,7 +333,14 @@ class accuinsight(Common):
             set_visual_csv_path(path_for_setting_visual_csv)
 
             csv_logger = CSVLogger(visual_csv_full_path, append=True, separator=';')
- 
+
+            # get train data(x) for computing shap value
+            if 'x':
+                x_train = x
+            if shap_on:
+                get_shap = shapCallbacks(x_train, feature_name, run_id, trigger=shap_on)
+            else:
+                pass
  
             ''' save json for visualization '''
             kwargs_dict = OrderedDict()
@@ -342,25 +372,25 @@ class accuinsight(Common):
             
             get_visual = visualCallbacks(x_validation=x_val, y_validation=y_val)
 
-
             if 'callbacks' in kwargs:
                 kwargs['callbacks'] += [csv_logger]
-                kwargs['callbacks'] += [get_visual]
-                kwargs['callbacks'] += [TrainHistoryCallbacks()]
-            
             else:
                 kwargs['callbacks'] = [csv_logger]
-                kwargs['callbacks'] += [get_visual]
-                kwargs['callbacks'] += [TrainHistoryCallbacks()]
 
-            fit_model = original(self, *args, **kwargs)
-            return(fit_model)
+            kwargs['callbacks'] += [get_visual]
+            if shap_on:
+                kwargs['callbacks'] += [get_shap]
+            else:
+                pass
+            kwargs['callbacks'] += [TrainHistoryCallbacks()]
+
+            return original(self, x, y, **kwargs)
 
         @gorilla.patch(keras.Model)
-        def fit(self, *args, **kwargs):
+        def fit(self, x, y, **kwargs):
             original = gorilla.get_original_attribute(keras.Model, 'fit')
             unlogged_params = ['self', 'x', 'y', 'callbacks', 'validation_data', 'verbose']
-            return run_and_log_function(self, original, args, kwargs, unlogged_params)
+            return run_and_log_function(self, original, x, y, kwargs, unlogged_params)
 
         settings = gorilla.Settings(allow_hit=True, store_hit=True)
         gorilla.apply(gorilla.Patch(keras.Model, 'fit', fit, settings=settings))
@@ -370,7 +400,7 @@ class accuinsight(Common):
         def stop_log(self, original, args, kwargs, unlogged_params):
             fit_model = original(self, *args, **kwargs)
             return(fit_model)
-        
+
         @gorilla.patch(keras.Model)
         def fit(self, *args, **kwargs):
             original = gorilla.get_original_attribute(keras.Model, 'fit')
