@@ -12,25 +12,24 @@ from Accuinsight.modeler.core import lossFuncInfo
 
 def model_type(model_name):
     if 'tensorflow' in str(model_name):
-        return('tf.keras')
+        return 'tf.keras'
     elif 'keras' in str(model_name):
-        return('keras')
+        return 'keras'
     elif 'state_dict' in dir(model_name):
-        return('pytorch')
-    
-    else:
-        if '(' in str(model_name):
-            tmp_name = str(model_name)[:str(model_name).find('(')]
-        elif '<' in str(model_name):
-            tmp_name = str(model_name).split('.')[0][1:]
+        return 'pytorch'
+
+    tmp_name = str(model_name)
+    if '(' in str(model_name):
+        tmp_name = str(model_name)[:str(model_name).find('(')]
+    elif '<' in str(model_name):
+        tmp_name = str(model_name).split('.')[0][1:]
+
+    if any(i in tmp_name for i in ONEVS):
+        idx1 = list(mit.locate(str(model_name), lambda x: x == "="))[0]
+        idx2 = list(mit.locate(str(model_name), lambda x: x == "("))[1]
+        return str(model_name)[idx1 + 1:idx2]
         
-        if any(i in tmp_name for i in ONEVS):
-            idx1 = list(mit.locate(str(model_name), lambda x: x == "="))[0]
-            idx2 = list(mit.locate(str(model_name), lambda x: x == "("))[1]
-            return(str(model_name)[idx1+1:idx2])
-        
-        else:
-            return(tmp_name)
+    return tmp_name
         
         
 def trial_number_all(DB_path, file_type):
@@ -54,7 +53,7 @@ def trial_number_all(DB_path, file_type):
         trial_num = 1
     
     os.chdir(r'..')                  ######## 상위 디렉토리로 이동 (경우에 따라서 변경 필요!!!!!) ########
-    return(str(trial_num))
+    return str(trial_num)
 
 def trial_num_json(DB_path, file_type):
     os.chdir(DB_path)
@@ -77,7 +76,7 @@ def trial_num_json(DB_path, file_type):
         trial_num = 1
     
     os.chdir(r'..')                  ######## 상위 디렉토리로 이동 (경우에 따라서 변경 필요!!!!!) ########
-    return(str(trial_num))
+    return str(trial_num)
 
 
 # model parameter
@@ -91,7 +90,6 @@ class from_model(object):
             self.keyList = list(model_name.params.keys())
             
     def all_params(self):
- 
         # multiclass
         try:
             parkey = [i for i in self.parDict.keys() if '__' in i]
@@ -121,7 +119,8 @@ class from_model(object):
             parkey.remove('random_state')
             return(parkey)
         else:
-            return(list(self.parDict.keys()))
+            return list(self.parDict.keys())
+
 
 def is_classification(model_name):
     if hasattr(model_name.loss, 'get_config'):      # tensorflow
